"""Turn BZ data into Prometheus data"""


class Promify:
    """Promify class"""
    def __init__(self, query, data):
        self.query = query
        self.data = data
        self.output = f"""# HELP {self.query["name"]} {self.query["help"]}
# TYPE {self.query["name"]} {self.query["type"]}
"""

    def promify(self):
        """Dispatch the correct formatting function"""
        # NOTE: We might need to be more picky here based on self.query['type']
        # but for now everything is just a count of matching data
        if 'group_field' in self.query:
            return self.promify_grouped_count()
        else:
            return self.promify_count()

    def promify_grouped_count(self):
        """Guages of grouped data"""
        group_field = self.query['group_field']
        counts = {}
        for item in self.data:
            if not item[group_field] in counts:
                counts[item[group_field]] = 1
            else:
                counts[item[group_field]] += 1

        for field in counts.keys():
            self.output += f"""{self.query['name']}{{{group_field}="{field}"}} {counts[field]}
"""

        return self.output

    def promify_count(self):
        """Simple counter"""
        count = len(self.data)
        self.output += f"""{self.query['name']} {count}
"""
        return self.output



"""Main class for PromBZEx"""
import os

from . import config
from . import server


class PromBZEx:
    """Main class"""
    config = None

    def __init__(self, config_file):
        self.config = config.PromBZExConfig(config_file)

    def config_get(self, key, server_name=None, allow_default=True):
        """Return a config item"""
        return self.config.get(key, server_name, allow_default)

    def servers(self):
        """Return the names of our servers"""
        return self.config.servers()

    def server(self, name):
        """Return an instance of BZServer"""
        return server.BZServer(name, self.config)

    def write(self, output, server_name=None):
        """Write output to a Prometheus file, possibly per-server"""
        filename = "bugzilla.prom"
        if server_name:
            filename = self.config_get('output_file', server_name)

        outfile = os.path.join(self.config_get('output_dir'), filename)
        with open(outfile, 'w') as filep:
            filep.write(output)



from unittest import mock
import pytest

from prombzex.prombzex import PromBZEx
from prombzex.server import BZServer
from prombzex.promify import Promify
from simple_rest_client.models import Response


def test_empty_config():
    with open('tests/config-empty.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert bz.servers() == []


def test_defaults_only():
    with open('tests/config-defaults-only.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert bz.servers() == []
    assert bz.config_get('timeout') == 12345


def test_one_server_no_api_key():
    with open('tests/config-one-server-no-api-key.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert len(bz.servers()) == 1
    for server_name in bz.servers():
        assert isinstance(server_name, str)
        with pytest.raises(KeyError):
            server = bz.server(server_name)


def test_one_server_no_queries():
    with open('tests/config-one-server-no-queries.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert len(bz.servers()) == 1
    for server_name in bz.servers():
        assert isinstance(server_name, str)
        server = bz.server(server_name)
        assert bz.config_get('timeout', server_name) == '657483'
        assert server.query_names() == []


def test_server_url_simple_counter(monkeypatch):
    def mock_search_return(params=None):
        return Response(status_code=200,
                        url='',
                        method='GET',
                        headers={},
                        client_response=None,
                        body={"bugs": [{}, {}]})

    with open('tests/config-one-server-one-url.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert len(bz.servers()) == 1
    for server_name in bz.servers():
        assert isinstance(server_name, str)
        server = bz.server(server_name)
        monkeypatch.setattr(server.bugzilla.bz, 'search', mock_search_return)
        with pytest.raises(KeyError):
            server.run_query('non-existant query')
        for query_name in server.query_names():
            output = server.run_query(query_name)
            assert output == """# HELP simple_counter_test Help for simple_counter_test
# TYPE simple_counter_test counter
simple_counter_test 2
"""


def test_server_query_simple_counter(monkeypatch):
    def mock_search_return(params=None):
        assert params['test1'] == 'test2'
        return Response(status_code=200,
                        url='',
                        method='GET',
                        headers={},
                        client_response=None,
                        body={"bugs": [{}, {}]})

    with open('tests/config-one-server-one-query.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)
    assert len(bz.servers()) == 1
    for server_name in bz.servers():
        assert isinstance(server_name, str)
        server = bz.server(server_name)
        monkeypatch.setattr(server.bugzilla.bz, 'search', mock_search_return)
        with pytest.raises(KeyError):
            server.run_query('non-existant query')
        for query_name in server.query_names():
            output = server.run_query(query_name)
            assert output == """# HELP simple_counter_test Help for simple_counter_test
# TYPE simple_counter_test counter
simple_counter_test 2
"""


def test_server_fail(monkeypatch):
    def mock_search_return(params=None):
        return Response(status_code=401,
                        url='',
                        method='GET',
                        headers={},
                        client_response=None,
                        body={})

    with open('tests/config-one-server-one-query.json', 'r') as filep:
        bz = PromBZEx(filep)
        server = bz.server(bz.servers()[0])
        monkeypatch.setattr(server.bugzilla.bz, 'search', mock_search_return)
        with pytest.raises(ValueError):
            server.run_query('simple_counter_test')


def test_output_writing(monkeypatch):
    import os
    import tempfile
    tempp, tmp_output_filename = tempfile.mkstemp()

    def mock_path_join(path, *paths):
        return tmp_output_filename

    with open('tests/config-one-server-one-query.json', 'r') as filep:
        bz = PromBZEx(filep)
    assert isinstance(bz, PromBZEx)

    test_output = "test1234"
    with monkeypatch.context() as m:
        m.setattr(os.path, 'join', mock_path_join)
        bz.write(test_output)
        bz.write(test_output, bz.servers()[0])

    with open(tmp_output_filename, 'r') as outp:
        written_output = outp.read()

    assert written_output == test_output
    os.remove(tmp_output_filename)


def test_promify_count():
    query = {}
    query["name"] = "test_name"
    query["help"] = "test_help"
    query["type"] = "test_type"
    data = [0, 1, 2, 3, 4, 5]

    obj = Promify(query, data)
    output = obj.promify()

    assert output == """# HELP test_name test_help
# TYPE test_name test_type
test_name 6
"""

def test_promify_grouped_count():
    query = {}
    query["name"] = "TestName"
    query["help"] = "TestHelp"
    query["type"] = "TestType"
    query["group_field"] = "TestGroup"
    data = [
        {"TestGroup": "groupA"},
        {"TestGroup": "groupB"},
        {"TestGroup": "groupC"},
        {"TestGroup": "groupA"},
    ]

    obj = Promify(query, data)
    output = obj.promify()

    assert output == """# HELP TestName TestHelp
# TYPE TestName TestType
TestName{TestGroup="groupA"} 2
TestName{TestGroup="groupB"} 1
TestName{TestGroup="groupC"} 1
"""



"""Scaffolding to expose the Bugzilla REST API nicely"""
import urllib.parse

from simple_rest_client.api import API
from simple_rest_client.resource import Resource

from . import promify


class BugzillaResource(Resource):
    """Class to define the custom REST endpoints we want"""
    actions = {
        'search': {'method': 'GET', 'url': 'rest/bug'},
    }


class BZServer:
    """Our object for interacting with a Bugzilla server"""
    def __init__(self, base_url, config):
        self.config = config
        data = self.config.server(base_url)

        self.base_url = base_url
        self.name = data["name"]
        self.api_key = data["api_key"]
        self.queries = data["queries"]

        self.bugzilla = API(
            api_root_url=self.base_url,
            params={"api_key": self.api_key},
            json_encode_body=True,
            timeout=self.config.get("timeout"),
        )
        self.bugzilla.add_resource(resource_name='bz',
                                   resource_class=BugzillaResource)

    def query_names(self):
        """Get the names of all queries defined for this server"""
        return [x["name"] for x in self.queries]

    def run_query(self, name):
        """Run a query on this server, by its name"""
        query_data = None
        for query in self.queries:
            if query["name"] == name:
                query_data = query
        if not query_data:
            raise KeyError

        params = {}
        if "params" in query_data:
            for param in query_data["params"]:
                params[param["key"]] = param["value"]
        elif "query" in query_data:
            url_parts = urllib.parse.urlparse(query_data["query"])
            params = urllib.parse.parse_qs(url_parts.query)
        response = self.bugzilla.bz.search(params=params)
        # print(response.status_code)
        # print(response.url)

        if response.status_code != 200:
            raise ValueError(response.body)

        prom = promify.Promify(query_data, response.body['bugs'])
        return prom.promify()



#!/usr/bin/env python3
"""Configuration for Prombzex"""

import json


class PromBZExConfig:
    """This class abstracts the json config file. Its format is:
    TODO: document the format"""

    data = None

    def __init__(self, config_file_object):
        """Initialise the class and load the config"""
        self.data = json.load(config_file_object)

    def servers(self):
        """Return a list of configured BZ servers"""
        return list(filter(lambda x: x != "default", self.data.keys()))

    def server(self, name):
        """Return the data for a server"""
        return self.data[name]

    def get(self, key, server=None, allow_default=True):
        """Return a config value for a given key, optionally for a
        specific server. If no server is specified, the default
        value is returned. If a server is specified, its value is
        returned, unless it defines no value, in which case the
        default is returned.
        If neither the specified server, nor the defaults, have a
        value for the key, None will be returned"""

        value = None

        if allow_default and 'default' in self.data and \
                key in self.data['default']:
            value = self.data['default'][key]

        if server and key in self.data[server]:
            value = self.data[server][key]

        return value



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='prombzex',
    version='0.2',
    author='Chris Jones',
    author_email='cmsj@tenshu.net',
    description='Prometheus Bugzilla Exporter',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/cmsj/prombzex',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    scripts=['bin/prombzex'],
    zip_safe=False
)
